<?php
/**
 * Skrill OxOrder
 *
 * The Class for Process Skrill Payment Gateways
 * Copyright (c) Skrill
 *
 * @class   SkrillOxOrder
 * @package Skrill/Classes
 * @located at /models
 * @extend  oxorder
 */
class SkrillOxOrder extends SkrillOxOrder_parent
{
    const ORDER_STORED                                      = 4;
    const WAITING_CLIENT_PAYMENT                            = 41;
    const AUTHORIZED                                        = 5;
    const AUTHORIZATION_WAITING                             = 51;
    const AUTHORIZATION_NOT_KNOWN                           = 52;
    const PAYMENT_REQUESTED                                 = 9;
    const PAYMENT_PROCESSING                                = 91;
    const PAYMENT_UNCERTAIN                                 = 92;

    /**
     * Order checking, processing and saving method.
     * Before saving performed checking if order is still not executed (checks in
     * database oxorder table for order with know ID), if yes - returns error code 3,
     * if not - loads payment data, assigns all info from basket to new oxorder object
     * and saves full order with error status. Then executes payment. On failure -
     * deletes order and returns error code 2. On success - saves order (oxorder::save()),
     * removes article from wishlist (oxorder::_updateWishlist()), updates voucher data
     * (oxorder::_markVouchers()). Finally sends order confirmation email to customer
     * (oxemail::SendOrderEMailToUser()) and shop owner (oxemail::SendOrderEMailToOwner()).
     * If this is order recalculation, skipping payment execution, marking vouchers as used
     * and sending order by email to shop owner and user
     * Mailing status (1 if OK, 0 on error) is returned.
     *
     * @param oxBasket $oBasket              Shopping basket object
     * @param object   $oUser                Current user object
     * @param bool     $blRecalculatingOrder Order recalculation
     * @param bool     $forceSave            Custom parameter to force save order
     *
     * @return integer
     */
    public function finalizeOrder(OxidEsales\Eshop\Application\Model\Basket $oBasket, $oUser, $blRecalculatingOrder = false, $forceSave = false)
    {
        $paymentId   = $oBasket->getPaymentId();
        // check if this order is already stored
        $sGetChallenge = oxRegistry::getSession()->getVariable('sess_challenge');

        if ($this->_checkOrderExist($sGetChallenge)) {
            oxRegistry::getUtils()->logger('BLOCKER');
            // we might use this later, this means that somebody clicked like mad on order button
            return self::ORDER_STATE_ORDEREXISTS;
        }

        // if not recalculating order, use sess_challenge id, else leave old order id
        if (!$blRecalculatingOrder) {
            // use this ID
            $this->setId($sGetChallenge);

            // validating various order/basket parameters before finalizing
            if ($iOrderState = $this->validateOrder($oBasket, $oUser)) {
                return $iOrderState;
            }
        }

        // copies user info
        $this->_setUser($oUser);

        // copies basket info
        $this->_loadFromBasket($oBasket);

        // payment information
        $oUserPayment = $this->_setPayment($oBasket->getPaymentId());

        if ((!$blRecalculatingOrder && strpos($paymentId, 'skrill') === false) or $forceSave) {
            $saveOrderResponse = $this->saveOrder($oBasket, $oUser, $blRecalculatingOrder, $oUserPayment);
            if ($saveOrderResponse != 'ACK') {
                return $saveOrderResponse;
            }
        }

        // store orderid
        $oBasket->setOrderId($this->getId());

        // updating wish lists
        $this->_updateWishlist($oBasket->getContents(), $oUser);

        // updating users notice list
        $this->_updateNoticeList($oBasket->getContents(), $oUser);

        // marking vouchers as used and sets them to $this->_aVoucherList (will be used in order email)
        // skipping this action in case of order recalculation
        if (!$blRecalculatingOrder) {
            $this->_markVouchers($oBasket, $oUser);
        }

        // send order by email to shop owner and current user
        // skipping this action in case of order recalculation
        if (!$blRecalculatingOrder && strpos($paymentId, 'skrill') === false) {
            $iRet = $this->_sendOrderByEmail($oUser, $oBasket, $oUserPayment);
        } else {
            $iRet = self::ORDER_STATE_OK;
        }
        
        return $iRet;
    }

    /**
     * Save order
     *
     * @param  oxBasket $oBasket              basket object
     * @param  object   $oUser                user object
     * @param  boolean  $blRecalculatingOrder recalculating or not
     * @param  object   $oUserPayment         user payment object
     * @return string|boolean
     */
    public function saveOrder(oxBasket $oBasket, $oUser, $blRecalculatingOrder = false, $oUserPayment = false)
    {
        // set folder information, if order is new
        // #M575 in recalculating order case folder must be the same as it was
        $this->_setFolderIfRecalculate($blRecalculatingOrder);

        // marking as not finished
        $this->_setOrderStatus('NOT_FINISHED');

        //saving all order data to DB
        $this->save();

        // executing payment (on failure deletes order and returns error code)
        // in case when recalculating order, payment execution is skipped
        // and executing TS protection
        if (!SkrillCore::isOxidSix()) {
            $blRet = $this->_executePaymentNProtection($blRecalculatingOrder, $oBasket, $oUserPayment);
            if ($blRet !== true) {
                return $blRet;
            }
        }

        // deleting remark info only when order is finished
        oxRegistry::getSession()->deleteVariable('ordrem');
        oxRegistry::getSession()->deleteVariable('stsprotection');

        if (!$this->oxorder__oxordernr->value) {
            $this->_setNumber();
        } else {
            oxNew('oxCounter')->update($this->_getCounterIdent(), $this->oxorder__oxordernr->value);
        }

        //#4005: Order creation time is not updated when order processing is complete
        if (!$blRecalculatingOrder) {
            $this->_updateOrderDate();
        }

        // updating order trans status (success status)
        $this->_setOrderStatus('OK');

        return 'ACK';
    }

    /**
     * Set folder if recalculate
     *
     * @param boolean $blRecalculatingOrder
     */
    protected function _setFolderIfRecalculate($blRecalculatingOrder = false)
    {
        // set folder information, if order is new
        // #M575 in recalculating order case folder must be the same as it was
        if (!$blRecalculatingOrder) {
            $this->_setFolder();
        }
    }

    /**
     * Execute payment and Execute TS protection
     *
     * @param  boolean $blRecalculatingOrder
     * @param  object  $oBasket              basket object
     * @param  object  $oUserPayment         user payment object
     * @return boolean
     */
    protected function _executePaymentNProtection($blRecalculatingOrder, $oBasket, $oUserPayment)
    {
        // executing payment (on failure deletes order and returns error code)
        // in case when recalculating order, payment execution is skipped
        if (!$blRecalculatingOrder) {
            $blRet = $this->_executePayment($oBasket, $oUserPayment);
            if ($blRet !== true) {
                return $blRet;
            }
        }

        // executing TS protection
        if (!$blRecalculatingOrder && $oBasket->getTsProductId()) {
            $blRet = $this->_executeTsProtection($oBasket);
            if ($blRet !== true) {
                return $blRet;
            }
        }
        return true;
    }

    /**
     * Check if order is using skrill
     *
     * @return boolean
     */
    public function isSkrillOrder()
    {
        $oBasket = oxRegistry::getSession()->getBasket();
        $paymentId = $oBasket->getPaymentId();
        return stristr($paymentId, 'skrill') !== false;
    }

    /**
     * Check if payment done
     *
     * @return boolean
     */
    public function isSkrillPaymentDone()
    {
        switch ($this->oxorder__skrill__status->value) {
            case self::ORDER_STORED:            // 4
            case self::WAITING_CLIENT_PAYMENT:  // 41
            case self::AUTHORIZED:              // 5
            case self::AUTHORIZATION_WAITING:   // 51
            case self::AUTHORIZATION_NOT_KNOWN: // 52
            case self::PAYMENT_REQUESTED:       // 9
            case self::PAYMENT_PROCESSING:      // 91
            case self::PAYMENT_UNCERTAIN:       // 92
                return true;
        }
        return false;
    }

    /**
     * Send emails
     */
    public function skrillSendEmails()
    {
        $order = oxNew('order');
        $oUser = $order->getUser();
        $oBasket = $order->getBasket();
        $oPayment = $order->getPayment();

        $orderState = $this->_sendOrderByEmail($oUser, $oBasket, $oPayment);

        return $orderState;
    }

    /**
     * Refresh order from basket
     *
     * @return object
     */
    public static function refreshOrder()
    {
        $newOrder = oxNew('skrilloxorder');
        $newOrder->load(oxRegistry::getSession()->getVariable('sess_challenge'));
        if (!$newOrder->oxorder__oxtotalordersum) {
            $newOrder->delete();
            $newOrder = oxNew('skrilloxorder');
            $order = oxNew('order');
            $_GET['sDeliveryAddressMD5'] = $order->getDeliveryAddressMD5();
            $basket = oxRegistry::getSession()->getBasket();
            $user = oxRegistry::getSession()->getUser();
            $orderState = $newOrder->finalizeOrder($basket, $user, true);
        }
        return $newOrder;
    }
}
